using AvaCorp.Virtual.Util;
using AvaCorp.Virtual.BLL;
using AvaCorp.Virtual.BLL.CTe.XML;
using AvaCorp.Virtual.BLL.NFe.XML;
using System.Xml;
using System;
using System.Text.RegularExpressions;

namespace AvaCorp.Virtual.WS
{
    /// <summary>
    /// Classe especializada para fazer a consulta do status do webservice da SEFAZ
    /// </summary>
    public partial class StatusServico
    {
        public XmlDocument ConsultaCompleta(string pChaveAcessoWebService, int pTipoDocumento, int pGrupo, int pEmpresa, int pFilial, int pUnidade)
        {
            XmlDocument retornoStatus = new XmlDocument();
            Serializacao serializacao = new Serializacao();
            string motivoRetorno = String.Empty;
            try
            {
                DALConnections.BeginTransaction();
                Auditoria.GravaMensagem(String.Format("ConsultaStatusServico\tChaveAcesso:{0}\tTipoDocumento:{1}\tGrupo:{2}\tEmpresa:{3}\tFilial:{4}\tUnidade:{5}", pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade));
                Configuracoes.GetParametros(pGrupo, pEmpresa, pFilial, pUnidade, pTipoDocumento);
                int servicoDocumentoEletronico = pTipoDocumento == 12 ? 33 : 15;
                BLL.Estado_ParametroDocumentoEletronico estadoBLL = new Estado_ParametroDocumentoEletronico();
                Types.Estado_ParametroDocumentoEletronico estadoParametroDocumentoEletronico = estadoBLL.Select("BRA", AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, servicoDocumentoEletronico);

                #region Sincroniza a situao do documento pelo EDIWS
                #region CTe v2.00/v3.00
                if (pTipoDocumento == 6 && (estadoParametroDocumentoEletronico.VersaoLayout == "2.00" || estadoParametroDocumentoEletronico.VersaoLayout == "3.00"))
                {
                    try
                    {
                        // Se vai fazer a transmisso pelo EDIWS, aborta a transao, pois ela no  mais necessria
                        DALConnections.RollbackTransaction();

                        var syncEDIWS = new Services.CTe.StatusServicoCTe();
                        var retSyscEDIWS = syncEDIWS.StatusServicoEDIWS(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade);

                        XmlDocument xmlSyndEDIWS = new XmlDocument();
                        xmlSyndEDIWS.LoadXml(string.Format("<EDIWS><descricaoResposta>{0}</descricaoResposta></EDIWS>", retSyscEDIWS.DescricaoRetorno));
                        return xmlSyndEDIWS;
                    }
                    catch (Exception ex)
                    {
                        string msg = string.Format("Problemas ao enviar documento pelo EDIWS: {0}", ex.Message);

                        XmlDocument xmlSyndEDIWS = new XmlDocument();
                        xmlSyndEDIWS.LoadXml(string.Format("<ERRO>{0}</ERRO>", msg));
                        return xmlSyndEDIWS;
                    }
                }
                #endregion
                #region NFe v3.10/v4.00
                if (pTipoDocumento == 12 && (estadoParametroDocumentoEletronico.VersaoLayout == "3.10" || estadoParametroDocumentoEletronico.VersaoLayout == "4.00"))
                {
                    try
                    {
                        // Se vai fazer a transmisso pelo EDIWS, aborta a transao, pois ela no  mais necessria
                        DALConnections.RollbackTransaction();

                        var syncEDIWS = new Services.NFe.StatusServicoNFe();
                        var retSyscEDIWS = syncEDIWS.StatusServicoEDIWS(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade);

                        XmlDocument xmlSyndEDIWS = new XmlDocument();
                        xmlSyndEDIWS.LoadXml(string.Format("<EDIWS><descricaoResposta>{0}</descricaoResposta></EDIWS>", retSyscEDIWS.DescricaoRetorno));
                        return xmlSyndEDIWS;
                    }
                    catch (Exception ex)
                    {
                        string msg = string.Format("Problemas ao enviar documento pelo EDIWS: {0}", ex.Message);

                        XmlDocument xmlSyndEDIWS = new XmlDocument();
                        xmlSyndEDIWS.LoadXml(string.Format("<ERRO>{0}</ERRO>", msg));
                        return xmlSyndEDIWS;
                    }
                }
                #endregion
                #endregion

                switch (pTipoDocumento)
                {
                    case 6:
                    case 97:
                        AvaCorp.Virtual.Services.CTe.StatusServicoCTe statusServicoCTe = new AvaCorp.Virtual.Services.CTe.StatusServicoCTe();
                        TRetConsStatServCTe retConsStatServCTe = statusServicoCTe.WSConsultaStatusServico(pTipoDocumento, estadoParametroDocumentoEletronico, pGrupo, pEmpresa);
                        retornoStatus = serializacao.GetMemoryXML(typeof(TRetConsStatServCTe), retConsStatServCTe);
                        break;
                    case 12:
                        AvaCorp.Virtual.Services.NFe.StatusServicoNFe statusServicoNFe = new AvaCorp.Virtual.Services.NFe.StatusServicoNFe();
                        TRetConsStatServNFe retConsStatServNFe = statusServicoNFe.WSConsultaStatusServico(AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, estadoParametroDocumentoEletronico, pGrupo, pEmpresa);
                        retornoStatus = serializacao.GetMemoryXML(typeof(TRetConsStatServNFe), retConsStatServNFe);
                        break;
                    default:
                        break;
                }
                motivoRetorno = Regex.Match(retornoStatus.InnerXml, @"<xMotivo>(?<motivo>.*)</xMotivo>", RegexOptions.Singleline | RegexOptions.IgnoreCase).Groups["motivo"].Value;
                Auditoria.GravaMensagem(motivoRetorno);
                DALConnections.CommitTransaction();

                return retornoStatus;
            }
            catch (Exception ex)
            {
                Auditoria.GravaMensagem(motivoRetorno);
                Auditoria.GravaMensagem(ex.Message);
                DALConnections.RollbackTransaction();
                LogSource.Log.GravarMensagem(pTipoDocumento, ex, 1, "Status servio", DALConnections.ConnectionString, motivoRetorno);
                return retornoStatus;
            }
        }
    }
}